/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.relm.ple.web.ui.audit.ProductAudit");

dojo.require('jazz.ui.StyledTable');
dojo.require("dojo.i18n");
dojo.requireLocalization("com.ibm.relm.ple.web.ui.audit", "AuditMessages");

dojo.require('jazz.util.html');
dojo.require('com.ibm.relm.common.web.ui.util');
dojo.require('dojo.date.locale');
dojo.require('jazz.ui.ResourceLink');

(function(){

	var util = com.ibm.relm.common.web.ui.util;
	var escape = jazz.util.html.escape;

	var messages = dojo.i18n.getLocalization("com.ibm.relm.ple.web.ui.audit", "AuditMessages");

	function newTd(withText, inRow, className) {
		var td = document.createElement('TD');
		util.setText(td, withText);
		if (inRow) inRow.appendChild(td);
		if (className) td.className = className;
		return td;
	}

	/**
	 * Formats the entry into a human readable string for the What or Change column.
	 * @return the human readable string. Can be HTML with links in it!
	 */
	function formatAuditChangeRow(auditEntry) {
		var type = auditEntry.type;

		var changeRow = dojo.create('tr');

		switch (type)
		{
			case 'addedProperty':
			case 'changedProperty':
			newTd(auditEntry.propertyName, changeRow, 'j-label');
			newTd(auditEntry.newValue, changeRow);
			break;
		
		case 'removedProperty':
			newTd(auditEntry.propertyName, changeRow, 'j-label');
			newTd(messages.removedPropertyValue, changeRow, 'j-label ple-ProductAudit-removed');
			break;
		
		case 'addedChild':
			newTd(messages.addedChild, changeRow, 'j-label');
			newTd(auditEntry.childName, changeRow);
			break;
		
		case 'removedChild':
			newTd(messages.removedChild, changeRow, 'j-label');
			newTd(auditEntry.childName, changeRow);
			break;
		
		case 'addedHyperlink':
			newTd(messages.addedHyperlink, changeRow, 'j-label');
			newTd('', changeRow).innerHTML = dojo.replace('<a target="_blank" href="{linkUrl}">{linkName}</a>', auditEntry);
			break;
		
		case 'removedHyperlink':
			newTd(messages.removedHyperlink, changeRow, 'j-label');
			newTd('', changeRow).innerHTML = dojo.replace('<a target="_blank" href="{linkUrl}">{linkName}</a>', auditEntry);
			break;
		
		case 'addedArtifact':
			newTd(messages.addedArtifact, changeRow, 'j-label');
			newTd('', changeRow).innerHTML = dojo.replace('<a class="jazz-ui-ResourceLink" target="_blank" href="{linkUrl}">{linkName}</a>', auditEntry);
			break;
		
		case 'removedArtifact':
			newTd(messages.removedArtifact, changeRow, 'j-label');
			newTd('', changeRow).innerHTML = dojo.replace('<a class="jazz-ui-ResourceLink" target="_blank" href="{linkUrl}">{linkName}</a>', auditEntry);
			break;
		
		case 'created':
			var createdString = auditEntry.isCreatedAutomatically ? messages.createdByByImplicitCheckOut : messages.createdByModification;
			newTd(createdString, changeRow, 'j-label');
			newTd('', changeRow);
			break;
		
		case 'snapshot':
			newTd(messages.addedToSnapshot, changeRow, 'j-label');
			newTd(auditEntry.snapshotName, changeRow);
			break;
		
		case 'changedPinned':
			var pinnedString = auditEntry.isPinned ? messages.pinnedChild : messages.unpinnedChild;
			newTd(pinnedString, changeRow, 'j-label');
			newTd(auditEntry.childName, changeRow);
			break;
		
		case 'boundChildChanged':
			newTd(messages.boundChild, changeRow, 'j-label');
			newTd(auditEntry.childName, changeRow);
			break;

		default:
			console.error('unknown audit entry', auditEntry);
			return type;
		}

		return changeRow;
	}

	dojo.declare(
		"com.ibm.relm.ple.web.ui.audit.ProductAudit",
		[ dijit._Widget ],
		{
			constructor: function(args, node)
			{
				this.auditData = args.auditData;
				this.resourceLinks = [];
				this._lazyFetchResourceLinks = args.lazyFetchResourceLinks;
			},

			postCreate: function() {
				this.inherited(arguments);
				dojo.addClass(this.domNode, 'ple-ProductAudit');

				// Must pre-sort the data. The table won't do it for us, it
				// just marks a column as what it's already sorted by .

				this.table = dojo.create('table', {
					className: 'ple-ProductAudit-table'
				}, this.domNode);

				this.tbody = dojo.create('tbody', {}, this.table);

				this._drawTable();
			},

			_destroyResourceLinks: function() {
				dojo.forEach(this.resourceLinks, function(link) {
					link.destroyRecursive();
				});

				this.resourceLinks.length = 0;
			},

			/**
			 * Empties the current table and recreates it from the current auditData.
			 * Will draw sorted in the order of the audit data.
			 */
			_drawTable: function() {
				this._destroyResourceLinks();
				dojo.empty(this.tbody);

				var frag = document.createDocumentFragment();

				var entriesLength = this.auditData.length;

				for (var i = 0; i < entriesLength; i++) {
					var auditEntry = this.auditData[i];

					var whoAndWhenRow = dojo.create('tr', {}, frag);

					// Who
					var whoCell = dojo.create('th', {
						innerHTML: auditEntry.who
					}, whoAndWhenRow);

					// When
					var dateString = dojo.date.locale.format(
						new Date(auditEntry.when),
						{ formatLength: 'medium' }
					);

					var whenCell = dojo.create('th', {
						className: 'ple-ProductAudit-when'
					}, whoAndWhenRow);
					util.setText(whenCell, dateString);

					var changesLength = auditEntry.changes.length;

					for (var j = 0; j < changesLength; j++) {
						var change = auditEntry.changes[j];

						var changeRow = formatAuditChangeRow(change);

						if (j === changesLength - 1) {
							dojo.query('td', changeRow).addClass('ple-ProductAudit-lastInChunk');
						}

						frag.appendChild(changeRow);

					}
				}

				// Now look for all links tagged with the ResourceLink class.
				// Convert all these links into rich hover ResorceLinks.
				// We must destroy these ourselves.
				var self = this;
				dojo.query('a.jazz-ui-ResourceLink', frag).forEach(function(plainLink) {
					var link = new jazz.ui.ResourceLink({
						lazyFetch: self._lazyFetchResourceLinks
					}, plainLink);
					self.resourceLinks.push(link);
				});

				this.tbody.appendChild(frag);
			},

			uninitialize: function() {
				this._destroyResourceLinks();
				this.inherited(arguments);
			}
		}
	);

})();
